// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information
use crate::{instruction::MOpRegNum, util::range_u32_nth_or_panic};
use fayalite::prelude::*;
use std::ops::Range;

#[hdl(cmp_eq)]
pub struct PowerIsaRegNum {
    pub value: UInt<5>,
}

#[hdl(cmp_eq)]
pub struct PowerIsaFRegNum {
    pub value: UInt<5>,
}

#[hdl(cmp_eq)]
pub struct PowerIsaCrFieldNum {
    pub value: UInt<3>,
}

#[hdl(cmp_eq)]
pub struct PowerIsaCrBitNum {
    pub cr_field: PowerIsaCrFieldNum,
    pub bit_in_field: UInt<2>,
}

impl MOpRegNum {
    pub const POWER_ISA_LR_REG_NUM: u32 = 1;
    pub const POWER_ISA_CTR_REG_NUM: u32 = 2;
    pub const POWER_ISA_TAR_REG_NUM: u32 = 3;
    /// XER bits are stored in [`PRegValue.flags`], bits that don't exist in [`PRegValue.flags`] are stored in [`PRegValue.int_fp`]
    ///
    /// [`PRegValue.flags`]: struct@crate::register::PRegValue
    /// [`PRegValue.int_fp`]: struct@crate::register::PRegValue
    pub const POWER_ISA_XER_REG_NUM: u32 = 4;

    pub const POWER_ISA_CR_REG_NUMS: Range<u32> = 8..16;
    pub const fn power_isa_cr_reg_num(index: usize) -> u32 {
        range_u32_nth_or_panic(&Self::POWER_ISA_CR_REG_NUMS, index)
    }

    pub const POWER_ISA_GPR_REG_NUMS: Range<u32> = 32..64;
    pub const fn power_isa_gpr_reg_num(index: usize) -> u32 {
        range_u32_nth_or_panic(&Self::POWER_ISA_GPR_REG_NUMS, index)
    }

    pub const POWER_ISA_FPR_REG_NUMS: Range<u32> = 64..96;
    pub const fn power_isa_fpr_reg_num(index: usize) -> u32 {
        range_u32_nth_or_panic(&Self::POWER_ISA_FPR_REG_NUMS, index)
    }
}
