// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information

pub mod tree_reduce;

pub(crate) const fn range_u32_len(range: &std::ops::Range<u32>) -> usize {
    let retval = range.end.saturating_sub(range.start);
    assert!(retval as usize as u32 == retval, "len overflowed");
    retval as usize
}

pub(crate) const fn range_u32_nth(range: &std::ops::Range<u32>, index: usize) -> Option<u32> {
    if index < range_u32_len(range) {
        Some(range.start + index as u32)
    } else {
        None
    }
}

pub(crate) const fn range_u32_nth_or_panic(range: &std::ops::Range<u32>, index: usize) -> u32 {
    if let Some(retval) = range_u32_nth(range, index) {
        retval
    } else {
        panic!("index out of range")
    }
}
